﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;

namespace Product_API_Availability_Request
{
    class Program
    {
        static void Main(string[] args)
        {
            HotelAvail();
        }

        static void HotelAvail()
        {
            // 1. Parameters (parameters you may specify on the request)

            string User = "InsertUsernameHere";
            string Pass = "InsertPasswordHere";
            string Language = "en";

            string DistributionString = "20"; // Adults and children repeated by the number of available rooms (Example: "2021", 2 room, first one of 2 adults and second one of 2 adults and 1 children)
            string ChildrenAgesString = ""; // 2 digit children ages (Example: "0815", 2 children, first one of 8 years old and second one of 15 years old).

            DateTime StartDate = DateTime.Now.AddDays(30);
            DateTime EndDate = DateTime.Now.AddDays(31);

            string Nationality = "ES";

            string JPDCode = "JPD040494"; // JPDCode for Madrid
            string HotelCode = string.Empty; // Through this node you may specify to search availability for an specific HotelCode. This request also allows for the availability request to be performed by multiple hotels codes (to do so you will simply need to add multiple HotelCode nodes within the HotelCodes node).

            string BoardType = string.Empty;

            // 2. Initialization (this code initializes all of the neccesaries nodes and propestier of the HotelAvail request)

            WebServiceJP.JP_HotelAvail HotelAvailRQ = new WebServiceJP.JP_HotelAvail();
            HotelAvailRQ.Version = "1.1";
            HotelAvailRQ.Language = Language;

            // 2.1. Login

            WebServiceJP.JP_Login Login = new WebServiceJP.JP_Login();
            Login.Email = User;
            Login.Password = Pass;
            HotelAvailRQ.Login = Login;

            // 2.2. HotelRequest

            WebServiceJP.JP_RequestHotelsAvail HotelRequest = new WebServiceJP.JP_RequestHotelsAvail();
            WebServiceJP.JP_SearchSegmentsHotels SearchSegmentsHotels = new WebServiceJP.JP_SearchSegmentsHotels();
            WebServiceJP.JP_SearchSegmentHotels SearchSegmentHotels = new WebServiceJP.JP_SearchSegmentHotels();
            SearchSegmentHotels.Start = StartDate;
            SearchSegmentHotels.End = EndDate;
            if (!string.IsNullOrEmpty(JPDCode))
                SearchSegmentHotels.JPDCode = JPDCode;
            SearchSegmentsHotels.SearchSegmentHotels = SearchSegmentHotels;
            SearchSegmentsHotels.CountryOfResidence = Nationality;
            if (!string.IsNullOrEmpty(HotelCode))
                SearchSegmentsHotels.HotelCodes = new[] { HotelCode };

            WebServiceJP.JP_Board[] Board;
            WebServiceJP.JP_Board Board1 = new WebServiceJP.JP_Board();
            if (!string.IsNullOrEmpty(BoardType))
                Board1.Type = BoardType;
            Board = new[] { Board1 };
            SearchSegmentsHotels.Boards = Board;

            HotelRequest.SearchSegmentsHotels = SearchSegmentsHotels;

            // 2.3. Passengers (Paxes/RelPaxesDist)

            // 2.3.1. Paxes: List of passengers and their ages
            WebServiceJP.JP_Paxes Paxes = new WebServiceJP.JP_Paxes();
            WebServiceJP.JP_Pax[] Pax;
            List<WebServiceJP.JP_Pax> PaxList = new List<WebServiceJP.JP_Pax>();

            // 2.3.2. RelPaxesDist: Distribution of passengers between rooms
            WebServiceJP.JP_HotelRelPaxDist[] RelPaxDist;
            List<WebServiceJP.JP_HotelRelPaxDist> RelPaxDistList = new List<WebServiceJP.JP_HotelRelPaxDist>();

            int numberOfRooms = DistributionString.Length / 2;
            int passenger = 1;
            int children = 1;
            for (int room = 1; room <= numberOfRooms; room += +1)
            {
                int adultsPerRoom = Int32.Parse(DistributionString.Substring(((room - 1) * 2), 1));
                int childrenPerRoom = Int32.Parse(DistributionString.Substring(((room - 1) * 2) + 1, 1));

                //RelPaxesDist (Each room)
                WebServiceJP.JP_HotelRelPaxDist RelPaxDist1 = new WebServiceJP.JP_HotelRelPaxDist();
                WebServiceJP.JP_RelPax[] RelPax;
                List<WebServiceJP.JP_RelPax> RelPaxList = new List<WebServiceJP.JP_RelPax>();

                for (int adult = 1; adult <= adultsPerRoom; adult += +1)
                {
                    //Paxes (List of all available paxes)
                    WebServiceJP.JP_Pax Pax1 = new WebServiceJP.JP_Pax();
                    Pax1.IdPax = passenger;
                    Pax1.AgeSpecified = true;
                    Pax1.Age = 30;
                    PaxList.Add(Pax1);

                    //RelPaxesDist (Passengers of each room)
                    WebServiceJP.JP_RelPax RelPax1 = new WebServiceJP.JP_RelPax();
                    RelPax1.IdPax = passenger;
                    RelPaxList.Add(RelPax1);

                    passenger += 1;
                }

                for (int nino = (adultsPerRoom + 1); nino <= (adultsPerRoom + childrenPerRoom); nino += +1)
                {
                    //Paxes (List of all available paxes)
                    WebServiceJP.JP_Pax Pax1 = new WebServiceJP.JP_Pax();
                    Pax1.IdPax = passenger;
                    Pax1.AgeSpecified = true;

                    if (ChildrenAgesString.Length < ((children - 1) * 2) + 2)
                        throw new System.ArgumentException("ChildrenAgesString has not been properly specified.");

                    Pax1.Age = Int32.Parse(ChildrenAgesString.Substring((children - 1) * 2, 2));

                    PaxList.Add(Pax1);

                    //RelPaxesDist (Passengers of each room)
                    WebServiceJP.JP_RelPax RelPax1 = new WebServiceJP.JP_RelPax();
                    RelPax1.IdPax = passenger;
                    RelPaxList.Add(RelPax1);

                    passenger += 1;
                    children += 1;
                }

                //RelPaxesDist (Room distribution being added to the room distribution list)
                RelPax = RelPaxList.ToArray();
                RelPaxDist1.RelPaxes = RelPax;
                RelPaxDistList.Add(RelPaxDist1);
            }

            //Paxes (List of all available paxes being added to the request)
            Pax = PaxList.ToArray();
            Paxes.Pax = Pax;
            HotelAvailRQ.Paxes = Paxes;

            //RelPaxesDist (Room distribution being added to the request)
            RelPaxDist = RelPaxDistList.ToArray();
            HotelRequest.RelPaxesDist = RelPaxDist;

            HotelAvailRQ.HotelRequest = HotelRequest;

            // 2.3. AdvancedOptions (optional parameters)

            WebServiceJP.JP_HotelAvailAdvancedOptions AdvancedOptions = new WebServiceJP.JP_HotelAvailAdvancedOptions();
            AdvancedOptions.ShowHotelInfoSpecified = true;
            AdvancedOptions.ShowHotelInfo = false;
            AdvancedOptions.ShowOnlyBestPriceCombinationSpecified = true;
            AdvancedOptions.ShowOnlyBestPriceCombination = true;
            AdvancedOptions.TimeOutSpecified = true;
            AdvancedOptions.TimeOut = 8000;

            HotelAvailRQ.AdvancedOptions = AdvancedOptions;

            // 3. HotelAvail Request (here we prepare and send the HotelAvail request)

            WebServiceJP.AvailTransactionsClient serv = new WebServiceJP.AvailTransactionsClient();

            Console.WriteLine(string.Format("- HOTELAVAIL REQUEST"));
            Console.WriteLine(string.Empty);
            Console.WriteLine(string.Format("Endpoint: {0}", serv.Endpoint.Address));
            Console.WriteLine(string.Format("User: {0}", User));
            Console.WriteLine(string.Format("Pass: {0}", Pass));
            Console.WriteLine(string.Format("Distribution: {0}", DistributionString));
            Console.WriteLine(string.Format("Dates: {0} - {1}", StartDate.ToShortDateString(), EndDate.ToShortDateString()));
            Console.WriteLine(string.Format("JPDCode: {0}", JPDCode));
            Console.WriteLine(string.Empty);

            WebServiceJP.JP_AvailResponseRS RST = serv.HotelAvail(HotelAvailRQ);

            // 4. HotelAvail Response (here we process the HotelAvail response)

            Console.WriteLine(string.Format("- HOTELAVAIL RESPONSE"));
            Console.WriteLine(string.Empty);
            if (RST != null && RST.Results != null)
            {
                // Dim HotelCount As Integer = 0
                int CombinationCount = 0;
                int HotelCount = 0;
                foreach (WebServiceJP.JP_HotelResult item in RST.Results.Items)
                {
                    HotelCount += 1;
                    foreach (WebServiceJP.JP_HotelOption hotelOption in item.HotelOptions)
                        CombinationCount += 1;
                }

                Console.WriteLine(string.Format("OK"));
                Console.WriteLine(string.Format("Hotels: {0}", HotelCount));
                Console.WriteLine(string.Format("Combinations: {0}", CombinationCount));
            }
            else if (RST != null && RST.Errors != null)
            {
                Console.WriteLine(string.Format("KO"));
                Console.WriteLine(string.Format("Error: {0}", RST.Errors[0].Code));
            }
            else
            {
                Console.WriteLine(string.Format("KO"));
                Console.WriteLine(string.Format("Error: UNKNOWN ERROR"));
            }

            // 5. XML serialization examples (for when you require for the XML logs any of the requests/reponses)

            string rqXML = serializeXML<WebServiceJP.JP_HotelAvail>(HotelAvailRQ);
            string rsXML = serializeXML<WebServiceJP.JP_AvailResponseRS>(RST);

            Console.ReadLine();
        }

        static string serializeXML<T>(object xmlobject)
        {
            XmlSerializer xsSubmit = new XmlSerializer(typeof(T));
            var subReq = xmlobject;

            using (var sww = new StringWriter())
            {
                using (XmlWriter writer = XmlWriter.Create(sww))
                {
                    xsSubmit.Serialize(writer, subReq);
                    return sww.ToString();
                }
            }
        }
    }
}
